﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Windows.Threading;

namespace Microsoft.Forums.ForumsClient.Models
{
    /// <summary>
    /// A custom ObservableCollection which allows edits and additions to be performed from
    /// background threads.  This class which automatically marshall these operations back to the ui thread
    /// </summary>
    /// <typeparam name="T">Type contained in collection</typeparam>
    public class DispatchingObservableCollection<T> : ObservableCollection<T>
    {

        /// <summary>
        /// Gets or sets the dipatcher UI.
        /// </summary>
        /// <value>The dipatcher UI.</value>
        public Dispatcher DispatcherUI
        {
            get;
            private set;
        }

        /// <summary>
        /// Constructor for DispatchingObservableCollection.  Takes a dispatcher for the ui thread.
        /// </summary>
        /// <param name="dispatcher">Dispatcher for the UI thread</param>
        public DispatchingObservableCollection(Dispatcher dispatcher)
        {
            if (dispatcher == null) throw new ArgumentNullException("dispatcher", "dispatcher must not be null");
            DispatcherUI = dispatcher;
        }

        /// <summary>
        /// Constructor for DispatchingObservableCollection.  Assumes current diaptcher is on ui thread.
        /// </summary>
        public DispatchingObservableCollection()
        {
            DispatcherUI = Dispatcher.CurrentDispatcher;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DispatchingObservableCollection&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="list">The list.</param>
        public DispatchingObservableCollection(List<T> list)
            : base(list)
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DispatchingObservableCollection&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="collection">The collection from which the elements are copied.</param>
        /// <exception cref="T:System.ArgumentNullException">
        /// The <paramref name="collection"/> parameter cannot be null.
        /// </exception>
        public DispatchingObservableCollection(IEnumerable<T> collection)
            : base(collection)
        {

        }

        /// <summary>
        /// Overrides default SetItem functionality 
        /// by marshalling calls back to the dispatcher thread
        /// </summary>
        /// <param name="index">Index of item</param>
        /// <param name="item">item to set</param>
        protected override void SetItem(int index, T item)
        {
            if (DispatcherUI.CheckAccess())
            {
                base.SetItem(index, item);
            }
            else
            {
                DispatcherUI.Invoke(
                    DispatcherPriority.Background,
                    (Action)delegate
                        {
                            SetItem(index, item);
                        });
            }
        }

        /// <summary>
        /// Overrides default RemoveItem functionality 
        /// by marshalling calls back to the dispatcher thread
        /// </summary>
        /// <param name="index">index of item to remove</param>
        protected override void RemoveItem(int index)
        {
            if (DispatcherUI.CheckAccess())
            {
                base.RemoveItem(index);
            }
            else
            {
                DispatcherUI.Invoke(
                    DispatcherPriority.Background,
                    (Action)delegate
                    {
                        RemoveItem(index);
                    });
            }
        }

        /// <summary>
        /// Overrides default ClearItems functionality 
        /// by marshalling calls back to the dispatcher thread
        /// </summary>
        protected override void ClearItems()
        {
            if (DispatcherUI.CheckAccess())
            {
                base.ClearItems();
            }
            else
            {
                DispatcherUI.Invoke(
                    DispatcherPriority.Background,
                    (Action)delegate
                    {
                        ClearItems();
                    });
            }
        }

        /// <summary>
        /// Overrides default InsertItem functionality 
        /// by marshalling calls back to the dispatcher thread
        /// </summary>
        /// <param name="index">index to insert</param>
        /// <param name="item">item to insert</param>
        protected override void InsertItem(int index, T item)
        {
            if (DispatcherUI.CheckAccess())
            {
                base.InsertItem(index, item);
            }
            else
            {
                DispatcherUI.Invoke(
                    DispatcherPriority.Background,
                    (Action)delegate
                    {
                        InsertItem(index, item);
                    });
            }
        }

        /// <summary>
        /// Overrides default MoveItem functionality 
        /// by marshalling calls back to the dispatcher thread
        /// </summary>
        /// <param name="oldIndex">index of item to move</param>
        /// <param name="newIndex">new index to mvoe item to</param>
        protected override void MoveItem(int oldIndex, int newIndex)
        {
            if (DispatcherUI.CheckAccess())
            {
                base.MoveItem(oldIndex, newIndex);
            }
            else
            {
                DispatcherUI.Invoke(
                    DispatcherPriority.Background,
                    (Action)delegate
                    {
                        MoveItem(oldIndex, newIndex);
                    });
            }
        }
    }
}
